@@ -36,6 +36,8 @@ class NetworkSysTypeEnum(Enum):  # TODO: move to reception sdk
     INPUT_KAFKA_BS0 = "INPUT_KAFKA_BS0"
     INPUT_HTTP_BS = "INPUT_HTTP_BS"
     INPUT_NEW_LK = "INPUT_NEW_LK"
+    INPUT_EXTERNAL_API = "INPUT_EXTERNAL_API"
+    INPUT_UNIVERSAL_API = "INPUT_UNIVERSAL_API"
 
     def __repr__(self) -> str:
         return f'{type(self).__name__}.{self.name}'
@@ -58,6 +60,7 @@ class ProtocolEnum(Enum):
     SMP_M_WATER_METER_12B_V0 = 'SMP_M_WATER_METER_12B_V0'
     SMP_M_WATER_METER_16B_V0 = 'SMP_M_WATER_METER_16B_V0'
     SMP_M_HEAT_PROXY_METER_16B_V0 = 'SMP_M_HEAT_PROXY_METER_16B_V0'
+    SMP_M_HEAT_GROUP_METER_V0 = 'SMP_M_HEAT_GROUP_METER_V0'
     WATER5_V_JUPITER_FREESCALE_V0 = 'WATER5_V_JUPITER_FREESCALE_V0'
     WATER5_V_JUPITER_STM_V0 = 'WATER5_V_JUPITER_STM_V0'
     WATER5_V_FLUO_STM_V0 = 'WATER5_V_FLUO_STM_V0'
@@ -68,6 +71,7 @@ class ProtocolEnum(Enum):
     WATER5_V_JUPITER_LORA_V0 = 'WATER5_V_JUPITER_LORA_V0'
     WATER5_V_FLUO_LORA_V0 = 'WATER5_V_FLUO_LORA_V0'
     SMP_M_INTERNAL_INFO_DATA = 'SMP_M_INTERNAL_INFO_DATA'
+    ARVAS_API_V0 = 'ARVAS_API_V0'
 
     def __repr__(self) -> str:
         return f'{type(self).__name__}.{self.name}'
@@ -130,6 +134,32 @@ class ResourceKind(Enum):
         return f'{type(self).__name__}.{self.name}'
 
 
+class SensorType(Enum):
+    """
+    Enumeration of sensor types for devices
+    """
+    ENERGY_METER_VOLTAGE = 'ENERGY_METER_VOLTAGE'
+    ENERGY_METER_CURRENT = 'ENERGY_METER_CURRENT'
+    ENERGY_METER_FREQUENCY = 'ENERGY_METER_FREQUENCY'
+    ENERGY_METER_POWER_FACTOR = 'ENERGY_METER_POWER_FACTOR'
+    TEMPERATURE = 'TEMPERATURE'
+    BATTERY = 'BATTERY'
+    UPTIME = 'UPTIME'
+
+    def __repr__(self) -> str:
+        return f'{type(self).__name__}.{self.name}'
+
+
+class JournalDataType(Enum):
+    """
+    Enumeration of journal data types
+    """
+    END_OF_DAY = 'END_OF_DAY'
+    END_OF_MONTH = 'END_OF_MONTH'
+    END_OF_YEAR = 'END_OF_YEAR'
+    CURRENT = 'CURRENT'
+
+
 class DeviceValueMarker(Enum):
     """
     Enumeration of markers for device values
@@ -310,11 +340,63 @@ class IntegrationV0MessageEvent(Enum):
     LOAD_RELAY_STATE_CHANGE = 'LOAD_RELAY_STATE_CHANGE'
     SIGNALIZATION_RELAY_STATE_CHANGE = 'SIGNALIZATION_RELAY_STATE_CHANGE'
 
+    SYSTEM__CIRCUIT_BREAK_T_SENSOR_1 = 'SYSTEM__CIRCUIT_BREAK_T_SENSOR_1'
+    SYSTEM__CIRCUIT_BREAK_T_SENSOR_2 = 'SYSTEM__CIRCUIT_BREAK_T_SENSOR_2'
+    SYSTEM__CIRCUIT_BREAK_T_SENSOR_3 = 'SYSTEM__CIRCUIT_BREAK_T_SENSOR_3'
+    SYSTEM__ERROR_DELTA_T = 'SYSTEM__ERROR_DELTA_T'
+    SYSTEM__CONSUMPTION_LOWER_G_MIN_CHANNEL_RATE_1 = 'SYSTEM__CONSUMPTION_LOWER_G_MIN_CHANNEL_RATE_1'
+    SYSTEM__CONSUMPTION_LOWER_G_MIN_CHANNEL_RATE_2 = 'SYSTEM__CONSUMPTION_LOWER_G_MIN_CHANNEL_RATE_2'
+    SYSTEM__CONSUMPTION_LOWER_G_MIN_CHANNEL_RATE_3 = 'SYSTEM__CONSUMPTION_LOWER_G_MIN_CHANNEL_RATE_3'
+    SYSTEM__CONSUMPTION_LOWER_G_MAX_CHANNEL_RATE_1 = 'SYSTEM__CONSUMPTION_LOWER_G_MAX_CHANNEL_RATE_1'
+    SYSTEM__CONSUMPTION_LOWER_G_MAX_CHANNEL_RATE_2 = 'SYSTEM__CONSUMPTION_LOWER_G_MAX_CHANNEL_RATE_2'
+    SYSTEM__CONSUMPTION_LOWER_G_MAX_CHANNEL_RATE_3 = 'SYSTEM__CONSUMPTION_LOWER_G_MAX_CHANNEL_RATE_3'
+    SYSTEM__NO_COOLANT_CHANNEL_RATE_1 = 'SYSTEM__NO_COOLANT_CHANNEL_RATE_1'
+    SYSTEM__NO_COOLANT_CHANNEL_RATE_2 = 'SYSTEM__NO_COOLANT_CHANNEL_RATE_2'
+    SYSTEM__NO_COOLANT_CHANNEL_RATE_3 = 'SYSTEM__NO_COOLANT_CHANNEL_RATE_3'
+    SYSTEM__CIRCUIT_BREAK_STIMULATION_CHANNEL_RATE_1 = 'SYSTEM__CIRCUIT_BREAK_STIMULATION_CHANNEL_RATE_1'
+    SYSTEM__CIRCUIT_BREAK_STIMULATION_CHANNEL_RATE_2 = 'SYSTEM__CIRCUIT_BREAK_STIMULATION_CHANNEL_RATE_2'
+    SYSTEM__CIRCUIT_BREAK_PRESSURE_SENSOR_1 = 'SYSTEM__CIRCUIT_BREAK_PRESSURE_SENSOR_1'
+    SYSTEM__CIRCUIT_BREAK_PRESSURE_SENSOR_2 = 'SYSTEM__CIRCUIT_BREAK_PRESSURE_SENSOR_2'
+    SYSTEM__CIRCUIT_BREAK_PRESSURE_SENSOR_3 = 'SYSTEM__CIRCUIT_BREAK_PRESSURE_SENSOR_3'
+    SYSTEM__REVERSE = 'SYSTEM__REVERSE'
+
+    DEVICE__POWER_SUPPLY_LOST = 'DEVICE__POWER_SUPPLY_LOST'
+    DEVICE__POWER_SUPPLY_ENABLED = 'DEVICE__POWER_SUPPLY_ENABLED'
+    DEVICE__COMMON_SETTINGS_CHANGED = 'DEVICE__COMMON_SETTINGS_CHANGED'
+    DEVICE__ALARM_DIGITAL_INPUT_1 = 'DEVICE__ALARM_DIGITAL_INPUT_1'
+    DEVICE__ALARM_OVERRATE_DIGITAL_INPUT_1 = 'DEVICE__ALARM_OVERRATE_DIGITAL_INPUT_1'
+    DEVICE__ALARM_OVERRATE_DIGITAL_INPUT_2 = 'DEVICE__ALARM_OVERRATE_DIGITAL_INPUT_2'
+    DEVICE__ALARM_UNDERRATE_DIGITAL_INPUT_1 = 'DEVICE__ALARM_UNDERRATE_DIGITAL_INPUT_1'
+    DEVICE__ALARM_UNDERRATE_DIGITAL_INPUT_2 = 'DEVICE__ALARM_UNDERRATE_DIGITAL_INPUT_2'
+    DEVICE__ALARM_T_HIGH_DIGITAL_INPUT_1 = 'DEVICE__ALARM_T_HIGH_DIGITAL_INPUT_1'
+    DEVICE__ALARM_T_HIGH_DIGITAL_INPUT_2 = 'DEVICE__ALARM_T_HIGH_DIGITAL_INPUT_2'
+    DEVICE__ALARM_T_LOW_DIGITAL_INPUT_1 = 'DEVICE__ALARM_T_LOW_DIGITAL_INPUT_1'
+    DEVICE__ALARM_T_LOW_DIGITAL_INPUT_2 = 'DEVICE__ALARM_T_LOW_DIGITAL_INPUT_2'
+    DEVICE__ALARM_DELTA_T_HIGH_DIGITAL_INPUT_1 = 'DEVICE__ALARM_DELTA_T_HIGH_DIGITAL_INPUT_1'
+    DEVICE__ALARM_DELTA_T_HIGH_DIGITAL_INPUT_2 = 'DEVICE__ALARM_DELTA_T_HIGH_DIGITAL_INPUT_2'
+    DEVICE__ALARM_DELTA_T_LOW_DIGITAL_INPUT_1 = 'DEVICE__ALARM_DELTA_T_LOW_DIGITAL_INPUT_1'
+    DEVICE__ALARM_DELTA_T_LOW_DIGITAL_INPUT_2 = 'DEVICE__ALARM_DELTA_T_LOW_DIGITAL_INPUT_2'
+    DEVICE__ALARM_POWER_HIGH_DIGITAL_INPUT_1 = 'DEVICE__ALARM_POWER_HIGH_DIGITAL_INPUT_1'
+    DEVICE__ALARM_POWER_HIGH_DIGITAL_INPUT_2 = 'DEVICE__ALARM_POWER_HIGH_DIGITAL_INPUT_2'
+    DEVICE__ALARM_POWER_LOW_DIGITAL_INPUT_1 = 'DEVICE__ALARM_POWER_LOW_DIGITAL_INPUT_1'
+    DEVICE__ALARM_POWER_LOW_DIGITAL_INPUT_2 = 'DEVICE__ALARM_POWER_LOW_DIGITAL_INPUT_2'
+    DEVICE__MEASURE_CHANNELS_SETTINGS_CHANGED = 'DEVICE__MEASURE_CHANNELS_SETTINGS_CHANGED'
+    DEVICE__SYSTEM_SETTINGS_CHANGED_1 = 'DEVICE__SYSTEM_SETTINGS_CHANGED_1'
+    DEVICE__SYSTEM_SETTINGS_CHANGED_2 = 'DEVICE__SYSTEM_SETTINGS_CHANGED_2'
+    DEVICE__SYSTEM_SETTINGS_CHANGED_3 = 'DEVICE__SYSTEM_SETTINGS_CHANGED_3'
+    DEVICE__SYSTEM_SETTINGS_CHANGED_4 = 'DEVICE__SYSTEM_SETTINGS_CHANGED_4'
+    DEVICE__DIGITAL_I_O_SETTINGS_CHANGED = 'DEVICE__DIGITAL_I_O_SETTINGS_CHANGED'
+    DEVICE__DATETIME_CHANGED = 'DEVICE__DATETIME_CHANGED'
+    DEVICE__ETHERNET_SETTINGS_CHANGED = 'DEVICE__ETHERNET_SETTINGS_CHANGED'
+
     def __repr__(self) -> str:
         return f'{type(self).__name__}.{self.name}'
 
 
 class IntegrationV0MessageMetaBSChannelProtocol(Enum):
+    """
+    Enumeration of base stations channel protocols
+    """
     nbfi = 'nbfi'
     unbp = 'unbp'
 
@@ -323,6 +405,9 @@ class IntegrationV0MessageMetaBSChannelProtocol(Enum):
 
 
 class IntervalSelectValue(Enum):
+    """
+    Enumeration of intervals
+    """
     half_hour = '30 minute'
     hour = '60 minutes'
     day = '1 day'
@@ -378,6 +463,9 @@ class DeviceClockOutOfSyncType(Enum):
 
 
 class DownlinkTaskType(Enum):
+    """
+    Enumeration of task types for downlink
+    """
     time_sync = 'time_sync'
     firmware_update = 'firmware_update'
     unbp_message = 'unbp_message'
@@ -387,6 +475,9 @@ class DownlinkTaskType(Enum):
 
 
 class ScheduleType(Enum):
+    """
+    Enumeration of schedule types
+    """
     on_capture = 'on_capture'
     schedule = 'schedule'
     asap = 'asap'
@@ -396,7 +487,9 @@ class ScheduleType(Enum):
 
 
 class ReglamentType(Enum):
-    ul_pr_dl_none = 'ul_pr_dl_none'
+    """
+    Enumeration of regulation types
+    """
     ul_pr_dl_rm_on_capture = 'ul_pr_dl_rm_on_capture'
     ul_pr_dl_rm = 'ul_pr_dl_rm'
     ul_rm_dl_rm = 'ul_rm_dl_rm'
@@ -406,13 +499,21 @@ class ReglamentType(Enum):
 
 
 class SignalModulation(Enum):
-    fsk = 'fsk'
+    """
+    Enumeration of signal modulations
+    """
+    DBPSK = 'DBPSK'
+    PSK = 'PSK'
+    FSK = 'FSK'
 
     def __repr__(self) -> str:
         return f'{type(self).__name__}.{self.name}'
 
 
 class DownlinkTaskStatus(Enum):
+    """
+    Enumeration of task statuses for downlink
+    """
     created = 'created'
     bs_ack = 'bs_ack'
     bs_executed_succeed = 'bs_executed_succeed'
@@ -426,6 +527,9 @@ class DownlinkTaskStatus(Enum):
 
 
 class DataAggregatorApiUserType(Enum):
+    """
+    Enumeration of ApiUser types for DataAggregator
+    """
     base_station = "base_station"
     api_user = "api_user"
     other = "other"
