@@ -1,17 +1,20 @@
 import base64
-from datetime import date, datetime, time
+import json
+from datetime import date, datetime
 from typing import Dict, Any, List, Tuple, Optional, Union
 from uuid import UUID
 
-from api_utils.api_resource.api_response import JsonApiResponsePayload
-from api_utils.api_resource.api_response_payload_alias import ApiBaseUserModelPayloadResponse, ApiBaseModelPayloadResponse
-from api_utils.internal_api.internal_api import InternalApi
-from api_utils.validators.custom_fields import WhiteSpaceStrippedStr, PgTypeInt32, CronScheduleStr
+from ul_api_utils.api_resource.api_response import JsonApiResponsePayload
+from ul_api_utils.api_resource.api_response_payload_alias import ApiBaseUserModelPayloadResponse, ApiBaseModelPayloadResponse
+from ul_api_utils.internal_api.internal_api import InternalApi
+from ul_api_utils.internal_api.internal_api_response import InternalApiResponse
+from ul_api_utils.validators.custom_fields import WhiteSpaceStrippedStr, PgTypeInt32, CronScheduleStr, UTCOffsetSeconds
+from ul_api_utils.utils.json_encoder import CustomJSONEncoder
 from pydantic import BaseModel, UUID4, Field
-from pydantic.types import NonNegativeInt
 
 from data_aggregator_sdk.constants.enums import IntervalSelectValue, EncryptionType, ResourceKind, NetworkTypeEnum, \
-    NetworkSysTypeEnum, DeviceHack, DataAggregatorApiUserType, DownlinkTaskType, ScheduleType, ReglamentType, SignalModulation
+    NetworkSysTypeEnum, DeviceHack, DataAggregatorApiUserType, DownlinkTaskType, ScheduleType, ReglamentType, \
+    SignalModulation, DownlinkTaskStatus, IntegrationV0MessageEvent, JournalDataType
 from data_aggregator_sdk.data_aggregator_api_sdk_config import DataAggregatorApiSdkConfig
 from data_aggregator_sdk.integration_message import ProfileKind, ProfileGranulation
 from data_aggregator_sdk.types.device import ApiImportDeviceResponse, ApiDataGatewayNetworkDevicePayloadResponse, \
@@ -190,6 +193,11 @@ class GeneralDeviceMeterValueDescriptionModel(BaseModel):
     registration_date: Optional[datetime] = Field(None, title='Registration Date', description='registration date')
     kind: ResourceKind = Field(ResourceKind.COMMON_CONSUMED, title='Kind', description=f'meter kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(-1, title='Tariff Number', description='tariff number')
+    journal_data_type: JournalDataType = Field(
+        JournalDataType.CURRENT,
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class DeviceBatteryLvlDescriptionModel(BaseModel):
@@ -220,6 +228,11 @@ class GeneralApiDeviceResponse(BaseModel):
     value: float = Field(title='Value', description='Value')
     kind: ResourceKind = Field(title='Kind', description=f'Kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(title='Tariff Number', description='tariff number')
+    journal_data_type: Optional[JournalDataType] = Field(
+        None,
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class GeneralApiDeviceMeterResponse(BaseModel):
@@ -228,6 +241,11 @@ class GeneralApiDeviceMeterResponse(BaseModel):
     value_date: datetime = Field(title='Value Date', description='Value Date')
     kind: ResourceKind = Field(title='Kind', description=f'Kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(title='Tariff Number', description='tariff number')
+    journal_data_type: Optional[JournalDataType] = Field(
+        None,
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class GeneralApiDeviceValueResponse(BaseModel):
@@ -239,6 +257,11 @@ class GeneralApiDeviceValueResponse(BaseModel):
     value_date: datetime = Field(title='Value Date', description='Value Date')
     last_value: Optional[float] = Field(None, title='Last Value', description='last value')
     last_value_date: Optional[datetime] = Field(None, title='Last Value Date', description='last value date')
+    journal_data_type: Optional[JournalDataType] = Field(
+        None,
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class GeneralApiDeviceMeterValueResponse(JsonApiResponsePayload):
@@ -249,6 +272,10 @@ class GeneralApiDeviceMeterValueResponse(JsonApiResponsePayload):
     value_date: datetime = Field(title='Value Date', description='Value Date')
     last_value: Optional[float] = Field(None, title='Last Value', description='last value')
     last_value_date: Optional[datetime] = Field(None, title='Last Value Date', description='last value date')
+    journal_data_type: JournalDataType = Field(
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class ApiDeviceLowBatteryResponse(JsonApiResponsePayload):
@@ -379,6 +406,10 @@ class ApiDeviceFirstValueDateResponse(JsonApiResponsePayload):
     value: float = Field(title='Value', description='value')
     kind: ResourceKind = Field(title='Kind', description=f'kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(title='Tariff Number', description='tariff number')
+    journal_data_type: JournalDataType = Field(
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class ApiMeterFirstValueDateResponse(JsonApiResponsePayload):
@@ -387,6 +418,10 @@ class ApiMeterFirstValueDateResponse(JsonApiResponsePayload):
     value_date: datetime = Field(title='Value Date', description='value date')
     kind: ResourceKind = Field(title='Kind', description=f'kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(title='Tariff Number', description='tariff number')
+    journal_data_type: JournalDataType = Field(
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class ApiLastValueEripReportsResponse(JsonApiResponsePayload):
@@ -396,6 +431,10 @@ class ApiLastValueEripReportsResponse(JsonApiResponsePayload):
     value: float = Field(title='Value', description='value')
     kind: ResourceKind = Field(title='Kind', description=f'kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(title='Tariff Number', description='tariff number')
+    journal_data_type: JournalDataType = Field(
+        title="Journal data type",
+        description=f"Journal data type ({', '.join(i.name for i in JournalDataType)})",
+    )
 
 
 class DeviceMac(JsonApiResponsePayload):
@@ -439,22 +478,23 @@ class ApiObjectValuesDeviceListResponse(JsonApiResponsePayload):
     objects: List[SelectValueResultObject] = Field(title='Objects', description='array of SelectValueResultObject')
 
 
-class DeviceEventsResponse(BaseModel):
+class DeviceEventsResponse(JsonApiResponsePayload):
     event_id: UUID4 = Field(title='Event Id', description='device event identifier')
-    date_created: str = Field(title='Date Created', description='date created')
+    date_created: datetime = Field(title='Date Created', description='date created')
     user_created_id: UUID4 = Field(title='User Created Id', description='user created id')
     value: Optional[float] = Field(None, title='Value', description='device event value')
     device_id: UUID4 = Field(title='Device Id', description='device identifier')
     serial_number: Optional[int] = Field(title='Serial Number', description='device channel serial number')
-    type: str = Field(title='Type', description='event type')
-    date: str = Field(title='Date', description='event date')
+    type: IntegrationV0MessageEvent = Field(title='Type', description='event type')
+    date: datetime = Field(title='Date', description='event date')
     data: Optional[Dict[str, Any]] = Field(None, title='Data', description='event extra data')
     is_system_generated: bool = Field(title='Is System Generated', description='is system generated')
 
 
-class DeviceStateSyncDataResponse(JsonApiResponsePayload):
-    device_events: List[DeviceEventsResponse] = Field(title='Device Events', description='array of DeviceEventsResponse')
-    offset_id: Optional[UUID4] = Field(None, title='Offset Id', description='Data State Sync Offset identifier')
+class ApiDeviceStateSyncDataResponse(JsonApiResponsePayload):
+    device_events: List[DeviceEventsResponse]
+    offset_id: Optional[UUID]
+    has_next: bool
 
 
 class UpdateSyncOffsetResponse(JsonApiResponsePayload):
@@ -497,6 +537,53 @@ class ApiBinaryDataResponse(JsonApiResponsePayload):
     file_name: str = Field(title='File Name', description='file name')
 
 
+class ApiDataGatewayNetworkDeviceForBsUplinkSignalResponse(JsonApiResponsePayload):
+    id: UUID4
+    date_created: datetime
+    date_modified: datetime
+    uplink_protocol_id: UUID4
+    downlink_protocol_id: UUID4
+    mac: int
+    key_id: Optional[UUID4]
+    device_id: UUID4
+    uplink_encryption_key: Optional[str]
+    downlink_encryption_key: Optional[str]
+    uplink_encryption_type: EncryptionType
+    downlink_encryption_type: EncryptionType
+
+
+class ApiDataGatewayNetworkDeviceShortResponse(JsonApiResponsePayload):
+    id: UUID4
+    mac: int
+
+
+class ApiBSUplinkSignalResponse(JsonApiResponsePayload):
+    id: UUID4
+    date_created: datetime
+    user_created_id: UUID4
+
+    rssi: Optional[int]
+    snr: Optional[int]
+    data_gateway_network_device_id: UUID
+
+    data_gateway_network_device: Optional[ApiDataGatewayNetworkDeviceForBsUplinkSignalResponse]
+    bs_id: UUID
+
+
+class BaseBSDownLinkStatusLogData(BaseModel):
+    effective_date: Optional[datetime]
+    bs_downlink_task_id: UUID
+    status: DownlinkTaskStatus
+    status_comment: str
+
+
+class ApiBSDownLinkStatusLogResponse(BaseBSDownLinkStatusLogData, JsonApiResponsePayload):
+    id: UUID4
+    is_active: bool
+    date_created: datetime
+    user_created_id: UUID4
+
+
 class BaseApiBSDownlinkTask(JsonApiResponsePayload):
     note: WhiteSpaceStrippedStr
     type: DownlinkTaskType
@@ -514,21 +601,51 @@ class BaseApiBSDownlinkTask(JsonApiResponsePayload):
     lbt_max_waiting_time_ms: int
     lbt_silent_time_ms: int
     force: bool
-    retry_count: NonNegativeInt
-    retry_min_delay_ms: int
     min_tx_delay_after_rx_ms: Optional[int]
     tx_duration_ms: Optional[int]
 
 
+class ApiBSDownlinkTaskTimeSyncResponse(BaseModel):
+    utc_offset_s: UTCOffsetSeconds
+
+
+class ApiBSDownlinkTaskUnbpMessageResponse(BaseModel):
+    payload: str
+    note: str
+
+
+class ApiBsDownlinkTaskFirmwareUpdateFileResponse(BaseModel):
+    files: List[ApiBinaryDataResponse]
+    sort_order: int
+    note: str
+
+
+class ApiBsDownlinkTaskFirmwareUpdateResponse(BaseModel):
+    bs_downlink_firmware_update_files: List[ApiBsDownlinkTaskFirmwareUpdateFileResponse]
+
+
+class ApiBsDownlinkTaskDataGatewayNetworkDeviceResponse(BaseModel):
+    data_gateway_network_device: ApiDataGatewayNetworkDeviceShortResponse
+
+
+class ApiBsDownlinkBaseStationResponse(BaseModel):
+    id: UUID4
+    base_station_api_user_id: UUID4
+
+
 class ApiBSDownlinkTaskResponse(BaseApiBSDownlinkTask):
     id: UUID4
+    bs: ApiBsDownlinkBaseStationResponse
     date_created: datetime
     user_created_id: UUID4
-    schedule_time: Optional[list[time]]
+    schedule_time: Optional[list[str]]
     schedule_cron: Optional[list[CronScheduleStr]]
-    bs_downlink_task_time_sync_id: Optional[UUID4]
-    bs_downlink_task_firmware_update_id: Optional[UUID4]
-    bs_downlink_task_unbp_message_id: Optional[UUID4]
+    bs_downlink_task_time_sync: Optional[ApiBSDownlinkTaskTimeSyncResponse]
+    bs_downlink_task_firmware_update: Optional[ApiBsDownlinkTaskFirmwareUpdateResponse]
+    bs_downlink_task_unbp_message: Optional[ApiBSDownlinkTaskUnbpMessageResponse]
+    current_status_log: ApiBSDownLinkStatusLogResponse
+    bs_downlink_task_dg_network_devices: List[ApiBsDownlinkTaskDataGatewayNetworkDeviceResponse]
+    is_active: Optional[bool] = None
 
 
 class ApiDataGatewayNetworkResponse(ApiBaseUserModelPayloadResponse):
@@ -550,6 +667,17 @@ class DataAggregatorApiUserResponse(ApiBaseModelPayloadResponse):
     data_gateway_network: Optional[ApiDataGatewayNetworkResponse] = Field(None, title='Data Gateway Network', description='Data Gateway Network')
 
 
+class ApiDeviceUptimeResponse(JsonApiResponsePayload):
+    id: UUID4 = Field(title='Identifier', description='uptime unique identifier')
+    date_created: datetime = Field(title='Date Created', description='date and time of record creation')
+    user_created_id: UUID4 = Field(title='User Created Identifier', description='api user created id')
+
+    device_id: UUID4 = Field(title='Device Identifier', description='device unique identifier')
+    channel_id: UUID4 = Field(title='Device Channel Identifier', description='device channel unique identifier')
+    date: datetime = Field(title='Date', description='date')
+    uptime_s: int = Field(title='Uptime value', description='uptime value seconds')
+
+
 class DataAggregatorApiSdk:
 
     def __init__(self, config: DataAggregatorApiSdkConfig) -> None:
@@ -586,7 +714,7 @@ class DataAggregatorApiSdk:
     '''used in services: Iot Account Application Backend'''
     @internal_api_error_handler
     def get_device_channels(self, device_id: UUID) -> List[ApiDeviceChannelPayloadResponse]:
-        return self._api_device.request_get(f'/device/{device_id}/device_channel/').typed(List[ApiDeviceChannelPayloadResponse]).check().payload
+        return self._api_device.request_get(f'/devices/{device_id}/device-channels').typed(List[ApiDeviceChannelPayloadResponse]).check().payload
 
     '''used in services: Iot Account Application Backend'''
     @internal_api_error_handler
@@ -757,7 +885,7 @@ class DataAggregatorApiSdk:
             body_data.update({'device_modification_type_id': str(device_modification_type_id)})
 
         return self._api_device.request_patch(
-            f'/device/{device_id}/factory-parameters',
+            f'/devices/{device_id}/factory-parameters',
             json=body_data,
         ).typed(ApiDeviceResponse).check().payload
 
@@ -779,12 +907,14 @@ class DataAggregatorApiSdk:
         iteration_interval: Optional[IntervalSelectValue],
         devices: GeneralDeviceValueModel,
         locf: Optional[bool],
+        journal_data_type: Optional[JournalDataType],
     ) -> List[ApiValueDeviceListResponse]:
         return self._api_device.request_post('/list-device/value-for-device-list', json=devices, q={
             'period_from': period_from,
             'period_to': period_to,
             'locf': locf,
             'iteration_interval': iteration_interval.value if iteration_interval is not None else None,
+            'journal_data_type': journal_data_type.value if journal_data_type is not None else None,
         }).typed(List[ApiValueDeviceListResponse]).check().payload
 
     '''used in services: Iot Account Application Backend'''
@@ -794,12 +924,14 @@ class DataAggregatorApiSdk:
         period_from: date | datetime,
         period_to: date | datetime,
         iteration_interval: Optional[IntervalSelectValue],
+        journal_data_type: Optional[JournalDataType],
         devices_meter: GeneralDeviceMeterValueModel,
     ) -> List[GeneralApiDeviceMeterValueResponse]:
         return self._api_device.request_post('/list-device/value-for-devices-meter', json=devices_meter, q={
             'period_from': period_from,
             'period_to': period_to,
             'iteration_interval': iteration_interval.value if iteration_interval is not None else None,
+            'journal_data_type': journal_data_type.value if journal_data_type is not None else None,
         }).typed(List[GeneralApiDeviceMeterValueResponse]).check().payload
 
     '''used in services: Iot Account Application Backend'''
@@ -1067,8 +1199,8 @@ class DataAggregatorApiSdk:
 
     '''used in services: Iot Account Application Backend'''
     @internal_api_error_handler
-    def get_events_to_sync_api_user_data(self) -> DeviceStateSyncDataResponse:
-        return self._api_device.request_get('/device-state-sync/events').typed(DeviceStateSyncDataResponse).check().payload
+    def get_device_event_sync_chunk(self) -> ApiDeviceStateSyncDataResponse:
+        return self._api_device.request_post('/device-state-sync-offsets/events').typed(ApiDeviceStateSyncDataResponse).check().payload
 
     '''used in services: Iot Account Application Backend'''
     @internal_api_error_handler
@@ -1214,7 +1346,7 @@ class DataAggregatorApiSdk:
         if profile_kinds:
             query.update({"profile_kinds": profile_kinds})
         return self._api_device.request_get(
-            f'/devices/{device_id}/device-profiles/granularies',
+            f'/devices/{device_id}/device-profiles/granularities',
             q=query,
         ).typed(List[ApiDeviceProfileGranularityResponse]).check().payload
 
@@ -1241,10 +1373,30 @@ class DataAggregatorApiSdk:
     @internal_api_error_handler
     def get_bs_downlink_tasks(
         self,
+        limit: Optional[int],
+        offset: Optional[int],
+        bs_ids: Optional[List[UUID]] = None,
+        only_active: Optional[bool] = None,
+        task_current_statuses: Optional[List[DownlinkTaskStatus]] = None,
     ) -> List[ApiBSDownlinkTaskResponse]:
+        filters = []
+        query: Dict[str, Any] = {}
         assert self._api_device is not None
+        if task_current_statuses:
+            query['task_current_statuses'] = ', '.join(task_current_statuses)  # type: ignore
+        if only_active:
+            query['only_active'] = only_active
+        if bs_ids:
+            filters.append({"name": "bs_id", "op": "in", "val": tuple(bs_ids)})
+        if limit is not None:
+            query['limit'] = limit
+        if offset is not None:
+            query['offset'] = offset
+        if filters:
+            filters = json.dumps(filters, cls=CustomJSONEncoder)  # type: ignore
         return self._api_device.request_get(
             '/bs-downlink-tasks',
+            q=query | {'filter': filters},
         ).typed(List[ApiBSDownlinkTaskResponse]).check().payload
 
     @internal_api_error_handler
@@ -1276,6 +1428,18 @@ class DataAggregatorApiSdk:
             f'/binary-data/{binary_data_id}',
         ).typed(ApiBinaryDataResponse).check().payload
 
+    @internal_api_error_handler
+    def get_bs_downlink_task_status_log(
+        self,
+        bs_downlink_task_id: UUID,
+    ) -> ApiBSDownLinkStatusLogResponse:
+        query: Dict[str, Any] = {"bs_downlink_task_id": bs_downlink_task_id}
+        assert self._api_device is not None
+        return self._api_device.request_get(
+            '/bs-downlink-tasks/status-log',
+            q=query,
+        ).typed(ApiBSDownLinkStatusLogResponse).check().payload
+
 #     '''used in services: not used'''
 #     @internal_api_error_handler
 #     def get_data_gateway_netowork_device_list(
@@ -1310,3 +1474,48 @@ class DataAggregatorApiSdk:
         return self._api_device.request_get(
             f'/api-users/{api_user_id}',
         ).typed(DataAggregatorApiUserResponse).check().payload
+
+    @internal_api_error_handler
+    def get_data_aggregator_uptime_by_device_id_and_channel_id(
+        self,
+        device_id: UUID,
+        channel_id: UUID,
+        limit: Optional[int],
+        offset: Optional[int],
+        filters: Optional[List[Dict[str, Any]]],
+        sorts: Optional[List[Tuple[str, str]]],
+    ) -> InternalApiResponse[List[ApiDeviceUptimeResponse]]:        # type: ignore # broken type checking
+        query_params: Dict[str, Any] = {
+            "filter": filters,
+            "sort": sorts,
+        }
+        if limit is not None:
+            query_params['limit'] = limit
+        if offset is not None:
+            query_params['offset'] = offset
+        return self._api_device.request_get(
+            f'/devices/{device_id}/channels/{channel_id}/uptimes',
+            q=query_params,
+        ).typed(List[ApiDeviceUptimeResponse]).check()
+
+    @internal_api_error_handler
+    def get_data_aggregator_uptime_by_device_id(
+        self,
+        device_id: UUID,
+        limit: Optional[int],
+        offset: Optional[int],
+        filters: Optional[List[Dict[str, Any]]],
+        sorts: Optional[List[Tuple[str, str]]],
+    ) -> InternalApiResponse[List[ApiDeviceUptimeResponse]]:        # type: ignore # broken type checking
+        query_params: Dict[str, Any] = {
+            "filter": filters,
+            "sort": sorts,
+        }
+        if limit is not None:
+            query_params['limit'] = limit
+        if offset is not None:
+            query_params['offset'] = offset
+        return self._api_device.request_get(
+            f'/devices/{device_id}/uptimes',
+            q=query_params,
+        ).typed(List[ApiDeviceUptimeResponse]).check()
