@@ -1,15 +1,17 @@
 from datetime import datetime
+from decimal import Decimal
 from enum import Enum, IntEnum, unique
-from ipaddress import IPv4Address
 from typing import Optional, List, Tuple, Dict, Any
 
-from api_utils.errors import ValidateApiError
 from pydantic import BaseModel, UUID4, root_validator, Field  # pylint: disable=no-name-in-module
+from ul_api_utils.errors import ValidateApiError
+from ul_api_utils.utils.api_method import ApiMethod
 from unipipeline.message.uni_message import UniMessage
 
-from data_aggregator_sdk.types.device import NetworkSysTypeEnum
 from data_aggregator_sdk.constants.enums import IntegrationV0MessageEvent, IntegrationV0MessageMetaBSChannelProtocol, \
-    ProtocolEnum, IntegrationV0MessageErrorType
+    ProtocolEnum, IntegrationV0MessageErrorType, DownlinkTaskStatus, JournalDataType, SensorType
+from data_aggregator_sdk.constants.enums import ResourceKind
+from data_aggregator_sdk.types.device import NetworkSysTypeEnum
 
 
 class ProfileGranulation(Enum):
@@ -201,6 +203,18 @@ class IntegrationV0MessageClock(BaseModel):
     )
 
 
+class IntegrationV0MessageUptime(BaseModel):
+    channel_id: int = Field(
+        1,
+        title="Channel ID",
+        description="Device channel Id",
+    )
+    uptime_s: float = Field(
+        title="Uptime seconds",
+        description="Uptime seconds value",
+    )
+
+
 class IntegrationV0MessageRelay(BaseModel):
     relay_id: int = Field(
         -1,
@@ -318,16 +332,21 @@ class IntegrationV0MessageConsumption(BaseModel):
         title="Channel",
         description="Consumption channel number for collecting info",
     )
-    value: float = Field(
+    value: Decimal = Field(
         title="Channel value",
         description="Consumption channel value",
     )
-    overloading_value: Optional[float] = Field(
+    overloading_value: Optional[Decimal] = Field(
         None,
         title="Overloading channel value",
         description="Consumption channel overloading value. "
                     "If overloading_value = None - value is fully precise - must replace current. if not None - overload potentially could be",
     )
+    journal_data_type: JournalDataType = Field(
+        JournalDataType.CURRENT,
+        title="Journal data type",
+        description="Consumption journal data type",
+    )
 
 
 class IntegrationV0MessageGeneration(BaseModel):
@@ -345,16 +364,21 @@ class IntegrationV0MessageGeneration(BaseModel):
         description="Generation resource type",
     )
 
-    value: float = Field(
+    value: Decimal = Field(
         title="Generation value",
         description="Generation channel value",
     )
-    overloading_value: Optional[float] = Field(
+    overloading_value: Optional[Decimal] = Field(
         None,
         title="Overloading value",
         description="Generation overloading value. "
                     "If overloading_value = None - value is fully precise - must replace current. if not None - overload potentially could be",
     )
+    journal_data_type: JournalDataType = Field(
+        JournalDataType.CURRENT,
+        title="Journal data type",
+        description="Generation journal data type",
+    )
 
 
 @unique
@@ -385,6 +409,11 @@ class IntegrationV0MessageCurrentTemperature(BaseModel):
         title="Temperature value",
         description="Float value of environment current temperature",
     )
+    sensor_id: int = Field(
+        -1,
+        title="Sensor id",
+        description="Integer value of temperature sensor id, default: -1",
+    )
 
 
 class IntegrationV0MessageMetaNbFiBS0(BaseModel):
@@ -534,18 +563,43 @@ class IntegrationV0MessageMetaNBIoT(BaseModel):
     )
 
 
+class IntegrationV0MessageSensor(BaseModel):
+    channel_id: Optional[int] = Field(
+        None,
+        title="Channel ID",
+        description="Device channel Id",
+    )
+    meter_resource_kind: Optional[ResourceKind] = Field(
+        None,
+        title="Meter Resource Kind",
+        description="Device Meter Resource Kind",
+    )
+    sensor_id: int = Field(
+        -1,
+        title="Sensor id",
+        description="Integer value of sensor id, default: -1",
+    )
+    sensor_type: SensorType = Field(
+        title="Sensor type",
+        description="Integer value of sensor type",
+    )
+    value: Decimal = Field(
+        title="Sensor value",
+        description="Decimal value of sensor value",
+    )
+
+
 class IntegrationV0MessageData(BaseModel):
     is_valid: bool = Field(
         True,  # TODO: remove default
         title="Valid message",
         description="Validation integration message data field",
     )
-    dt: Optional[datetime] = Field(  # TODO: remove optional
-        None,  # TODO: remove default
+    dt: datetime = Field(
         title="Datetime message",
         description="Datetime integration message field which shows when message was received",
     )
-    battery: List[IntegrationV0MessageCurrentBatteryLevel] = Field(
+    battery: List[IntegrationV0MessageCurrentBatteryLevel] = Field(  # ToDo: move parameter to sensors
         default_factory=list,
         title="Battery info",
         description="Structure which contains battery ID and voltage level for current device",
@@ -565,12 +619,17 @@ class IntegrationV0MessageData(BaseModel):
         title="Profiles info",
         description="Structure which contains profiles info for current device",
     )
+    sensors: List[IntegrationV0MessageSensor] = Field(
+        default_factory=list,
+        title="Device sensors info",
+        description="Structure which contains sensors info for current device",
+    )
     events: List[IntegrationV0MessageEvent] = Field(
         default_factory=list,
         title="Events list",
         description="Events list which contains triggered events info for current device",
     )
-    temperature: List[IntegrationV0MessageCurrentTemperature] = Field(
+    temperature: List[IntegrationV0MessageCurrentTemperature] = Field(  # ToDo: move parameter to sensors
         default_factory=list,
         title="Temperature info",
         description="Structure which contains temperature info for current device",
@@ -585,6 +644,11 @@ class IntegrationV0MessageData(BaseModel):
         title="Clock info",
         description="Structure which contains clock info for current device",
     )
+    uptime: List[IntegrationV0MessageUptime] = Field(  # ToDo: move parameter to sensors
+        default_factory=list,
+        title="Uptime info",
+        description="Structure which contains uptime info for current device",
+    )
 
 
 class IntegrationMessageMetaIotAccountGateway(BaseModel):
@@ -615,6 +679,55 @@ class IntegrationMessageMetaRecycle(BaseModel):
     )
 
 
+class IntegrationMessageMetaExternalApiDataInput(BaseModel):
+    name: str = Field(
+        ...,
+        title='Api Integration name',
+        description='Api Integration name',
+    )
+    uri: str = Field(
+        ...,
+        title='External api uri',
+        description='External api uri',
+    )
+    path: str = Field(
+        ...,
+        title='External api path',
+        description='External api path',
+    )
+    api_method: ApiMethod = Field(
+        ...,
+        title='Api Method',
+        description='Api Method',
+    )
+    query_params: Optional[Dict[str, str]] = Field(
+        None,
+        title='Query Parameters',
+        description='Query Parameters',
+    )
+    status_code: int = Field(
+        ...,
+        title='Request status code',
+        description='Request status code',
+    )
+    body: Optional[str] = Field(
+        None,
+        title='Request Body',
+        description='Request Body',
+    )
+    headers: Optional[Dict[str, str]] = Field(
+        None,
+        title='Request Headers',
+        description='request headers',
+
+    )
+    current_dt: datetime = Field(
+        ...,
+        title="request response date and time",
+        description="The current date and time of the sender at the time of sending",
+    )
+
+
 class IntegrationMessageMetaUniversalDataInput(BaseModel):
     user_id: UUID4 = Field(
         ...,
@@ -661,7 +774,7 @@ class IntegrationMessageMetaUniversalDataInput(BaseModel):
         title="Note about sender",
         description="Any usefull information about sender",
     )
-    ipv4: IPv4Address = Field(
+    ipv4: str = Field(
         ...,
         title='IPv4 sender address',
         description="A unique numerical identifier for every sender that send through the internet",
@@ -689,6 +802,11 @@ class IntegrationV0MessageMeta(BaseModel):
         title="Universal data input meta",
         description="Universal data input meta structure",
     )
+    external_api_data_input: Optional[IntegrationMessageMetaExternalApiDataInput] = Field(
+        None,
+        title="External api data input meta",
+        description="External api data input meta structure",
+    )
     iot_account_gateway: Optional[IntegrationMessageMetaIotAccountGateway] = Field(
         None,
         title="IoT account gateway",
@@ -706,6 +824,7 @@ class IntegrationV0MessageMeta(BaseModel):
             values.get('nbfi_bs0'),
             values.get('nbiot'),
             values.get('bs_http'),
+            values.get('external_api_data_input'),
             values.get('universal_data_input'),
             values.get('iot_account_gateway'),
             values.get('recycle'),
@@ -822,3 +941,61 @@ class IntegrationV0MessageGateway(UniMessage):
         title="Error info",
         description="Error structure contains error type and error message string",
     )
+
+
+INTEGRATION_MESSAGE_RESOURCE_TYPE_AND_COUNTER_TYPE__TO__CONSUMED_RESOURCE_KIND: Dict[Tuple[ResourceType, CounterType], ResourceKind] = {
+    (ResourceType.COMMON, CounterType.ENERGY_PHASE_A): ResourceKind.PHASE_A_ACTIVE_CONSUMED,  # TODO: for uiversal api tmp resolution
+    (ResourceType.COMMON, CounterType.ENERGY_PHASE_B): ResourceKind.PHASE_B_ACTIVE_CONSUMED,  # TODO: for uiversal api tmp resolution
+    (ResourceType.COMMON, CounterType.ENERGY_PHASE_C): ResourceKind.PHASE_C_ACTIVE_CONSUMED,  # TODO: for uiversal api tmp resolution
+    (ResourceType.COMMON, CounterType.COMMON): ResourceKind.COMMON_CONSUMED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.COMMON): ResourceKind.COMMON_ACTIVE_CONSUMED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.COMMON): ResourceKind.COMMON_REACTIVE_CONSUMED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.ENERGY_PHASE_A): ResourceKind.PHASE_A_ACTIVE_CONSUMED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.ENERGY_PHASE_A): ResourceKind.PHASE_A_REACTIVE_CONSUMED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.ENERGY_PHASE_B): ResourceKind.PHASE_B_ACTIVE_CONSUMED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.ENERGY_PHASE_B): ResourceKind.PHASE_B_REACTIVE_CONSUMED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.ENERGY_PHASE_C): ResourceKind.PHASE_C_ACTIVE_CONSUMED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.ENERGY_PHASE_C): ResourceKind.PHASE_C_REACTIVE_CONSUMED,
+}
+INTEGRATION_MESSAGE_RESOURCE_TYPE_AND_COUNTER_TYPE__TO__GENERATED_RESOURCE_KIND: Dict[Tuple[ResourceType, CounterType], ResourceKind] = {
+    (ResourceType.COMMON, CounterType.ENERGY_PHASE_A): ResourceKind.COMMON_ACTIVE_GENERATED,  # TODO: for uiversal api tmp resolution
+    (ResourceType.COMMON, CounterType.ENERGY_PHASE_B): ResourceKind.COMMON_ACTIVE_GENERATED,  # TODO: for uiversal api tmp resolution
+    (ResourceType.COMMON, CounterType.ENERGY_PHASE_C): ResourceKind.COMMON_ACTIVE_GENERATED,  # TODO: for uiversal api tmp resolution
+    (ResourceType.COMMON, CounterType.COMMON): ResourceKind.COMMON_GENERATED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.COMMON): ResourceKind.COMMON_ACTIVE_GENERATED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.COMMON): ResourceKind.COMMON_REACTIVE_GENERATED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.ENERGY_PHASE_A): ResourceKind.PHASE_A_ACTIVE_GENERATED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.ENERGY_PHASE_A): ResourceKind.PHASE_A_REACTIVE_GENERATED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.ENERGY_PHASE_B): ResourceKind.PHASE_B_ACTIVE_GENERATED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.ENERGY_PHASE_B): ResourceKind.PHASE_B_REACTIVE_GENERATED,
+    (ResourceType.ENERGY_ACTIVE, CounterType.ENERGY_PHASE_C): ResourceKind.PHASE_C_ACTIVE_GENERATED,
+    (ResourceType.ENERGY_REACTIVE, CounterType.ENERGY_PHASE_C): ResourceKind.PHASE_C_REACTIVE_GENERATED,
+}
+
+
+class ManagementInputV0MessageData(BaseModel):
+    effective_date: datetime = Field(
+        title="Effective date",
+        description="Effective date",
+    )
+    bs_downlink_task_id: UUID4 = Field(
+        title="Task ID",
+        description="Task ID",
+    )
+    status: DownlinkTaskStatus = Field(
+        title="Downlink task status",
+        description="Downlink task status",
+    )
+    status_comment: Optional[str] = Field(
+        None,
+        title="Status comments",
+        description="Issue status comments",
+    )
+    user_created_id: UUID4 = Field(
+        title="User ID",
+        description="ID of the user who created the changes",
+    )
+
+
+class ManagementInputV0Message(UniMessage):
+    data: List[ManagementInputV0MessageData]
